@@ -1,137 +1,96 @@
 /*
  *************************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1996-2001, International Business Machines Corporation and
+ * Copyright (c) 1996-2010, International Business Machines Corporation and
  * others. All Rights Reserved.
  *************************************************************************
  */
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_NORMALIZATION
+
+#include "unicode/uniset.h"
 #include "unicode/unistr.h"
 #include "unicode/chariter.h"
 #include "unicode/schriter.h"
 #include "unicode/uchriter.h"
-#include "unicode/uiter.h"
 #include "unicode/normlzr.h"
 #include "cmemory.h"
-#include "unormimp.h"
+#include "normalizer2impl.h"
+#include "uprops.h"  // for uniset_getUnicode32Instance()
 
 U_NAMESPACE_BEGIN
 
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(Normalizer)
+
 //-------------------------------------------------------------------------
 // Constructors and other boilerplate
 //-------------------------------------------------------------------------
 
 Normalizer::Normalizer(const UnicodeString& str, UNormalizationMode mode) :
-    fUMode(mode), fOptions(0),
+    UObject(), fFilteredNorm2(NULL), fNorm2(NULL), fUMode(mode), fOptions(0),
+    text(new StringCharacterIterator(str)),
     currentIndex(0), nextIndex(0),
     buffer(), bufferPos(0)
 {
-    init(new StringCharacterIterator(str));
+    init();
 }
 
 Normalizer::Normalizer(const UChar *str, int32_t length, UNormalizationMode mode) :
-    fUMode(mode), fOptions(0),
+    UObject(), fFilteredNorm2(NULL), fNorm2(NULL), fUMode(mode), fOptions(0),
+    text(new UCharCharacterIterator(str, length)),
     currentIndex(0), nextIndex(0),
     buffer(), bufferPos(0)
 {
-    init(new UCharCharacterIterator(str, length));
+    init();
 }
 
 Normalizer::Normalizer(const CharacterIterator& iter, UNormalizationMode mode) :
-    fUMode(mode), fOptions(0),
+    UObject(), fFilteredNorm2(NULL), fNorm2(NULL), fUMode(mode), fOptions(0),
+    text(iter.clone()),
     currentIndex(0), nextIndex(0),
     buffer(), bufferPos(0)
 {
-    init(iter.clone());
-}
-
-// deprecated constructors
-
-Normalizer::Normalizer(const UnicodeString& str, 
-                       EMode mode) :
-    fUMode(getUMode(mode)), fOptions(0),
-    currentIndex(0), nextIndex(0),
-    buffer(), bufferPos(0)
-{
-    init(new StringCharacterIterator(str));
-}
-
-Normalizer::Normalizer(const UnicodeString& str, 
-                       EMode mode, 
-                       int32_t options) :
-    fUMode(getUMode(mode)), fOptions(options),
-    currentIndex(0), nextIndex(0),
-    buffer(), bufferPos(0)
-{
-    init(new StringCharacterIterator(str));
-}
-
-Normalizer::Normalizer(const UChar *str, int32_t length, EMode mode) :
-    fUMode(getUMode(mode)), fOptions(0),
-    currentIndex(0), nextIndex(0),
-    buffer(), bufferPos(0)
-{
-    init(new UCharCharacterIterator(str, length));
-}
-
-Normalizer::Normalizer(const CharacterIterator& iter, 
-                       EMode mode) :
-    fUMode(getUMode(mode)), fOptions(0),
-    currentIndex(0), nextIndex(0),
-    buffer(), bufferPos(0)
-{
-    init(iter.clone());
-}
-
-Normalizer::Normalizer(const CharacterIterator& iter, 
-                       EMode mode, 
-                       int32_t options) :
-    fUMode(getUMode(mode)), fOptions(options),
-    currentIndex(0), nextIndex(0),
-    buffer(), bufferPos(0)
-{
-    init(iter.clone());
+    init();
 }
 
 Normalizer::Normalizer(const Normalizer &copy) :
-    fUMode(copy.fUMode), fOptions(copy.fOptions),
+    UObject(copy), fFilteredNorm2(NULL), fNorm2(NULL), fUMode(copy.fUMode), fOptions(copy.fOptions),
+    text(copy.text->clone()),
     currentIndex(copy.currentIndex), nextIndex(copy.nextIndex),
     buffer(copy.buffer), bufferPos(copy.bufferPos)
 {
-    init(((CharacterIterator *)(copy.text->context))->clone());
+    init();
 }
 
 static const UChar _NUL=0;
 
 void
-Normalizer::init(CharacterIterator *iter) {
+Normalizer::init() {
     UErrorCode errorCode=U_ZERO_ERROR;
-
-    text=new UCharIterator;
-
-    if(unorm_haveData(&errorCode)) {
-        uiter_setCharacterIterator(text, iter);
-    } else {
-        delete iter;
-        uiter_setCharacterIterator(text, new UCharCharacterIterator(&_NUL, 0));
+    fNorm2=Normalizer2Factory::getInstance(fUMode, errorCode);
+    if(fOptions&UNORM_UNICODE_3_2) {
+        delete fFilteredNorm2;
+        fNorm2=fFilteredNorm2=
+            new FilteredNormalizer2(*fNorm2, *uniset_getUnicode32Instance(errorCode));
+    }
+    if(U_FAILURE(errorCode)) {
+        errorCode=U_ZERO_ERROR;
+        fNorm2=Normalizer2Factory::getNoopInstance(errorCode);
     }
 }
 
 Normalizer::~Normalizer()
 {
-    delete (CharacterIterator *)text->context;
+    delete fFilteredNorm2;
     delete text;
 }
 
 Normalizer* 
 Normalizer::clone() const
 {
-    if(this!=0) {
-        return new Normalizer(*this);
-    } else {
-        return 0;
-    }
+    return new Normalizer(*this);
 }
 
 /**
@@ -139,7 +98,7 @@ Normalizer::clone() const
  */
 int32_t Normalizer::hashCode() const
 {
-    return ((CharacterIterator *)(text->context))->hashCode() + fUMode + fOptions + buffer.hashCode() + bufferPos + currentIndex + nextIndex;
+    return text->hashCode() + fUMode + fOptions + buffer.hashCode() + bufferPos + currentIndex + nextIndex;
 }
     
 UBool Normalizer::operator==(const Normalizer& that) const
@@ -148,7 +107,7 @@ UBool Normalizer::operator==(const Normalizer& that) const
         this==&that ||
         fUMode==that.fUMode &&
         fOptions==that.fOptions &&
-        *((CharacterIterator *)(text->context))==*((CharacterIterator *)(that.text->context)) &&
+        *text==*that.text &&
         buffer==that.buffer &&
         bufferPos==that.bufferPos &&
         nextIndex==that.nextIndex;
@@ -158,7 +117,7 @@ UBool Normalizer::operator==(const Normalizer& that) const
 // Static utility methods
 //-------------------------------------------------------------------------
 
-void 
+void U_EXPORT2
 Normalizer::normalize(const UnicodeString& source, 
                       UNormalizationMode mode, int32_t options,
                       UnicodeString& result, 
@@ -169,107 +128,81 @@ Normalizer::normalize(const UnicodeString& source,
             status=U_ILLEGAL_ARGUMENT_ERROR;
         }
     } else {
-        UChar *buffer=result.getBuffer(source.length());
-        int32_t length=unorm_internalNormalize(buffer, result.getCapacity(),
-                                               source.getBuffer(), source.length(),
-                                               mode, (options&IGNORE_HANGUL)!=0,
-                                               &status);
-        result.releaseBuffer(length);
-        if(status==U_BUFFER_OVERFLOW_ERROR) {
-            status=U_ZERO_ERROR;
-            buffer=result.getBuffer(length);
-            length=unorm_internalNormalize(buffer, result.getCapacity(),
-                                           source.getBuffer(), source.length(),
-                                           mode, (options&IGNORE_HANGUL)!=0,
-                                           &status);
-            result.releaseBuffer(length);
-        }
+        UnicodeString localDest;
+        UnicodeString *dest;
 
-        if(U_FAILURE(status)) {
-            result.setToBogus();
+        if(&source!=&result) {
+            dest=&result;
+        } else {
+            // the source and result strings are the same object, use a temporary one
+            dest=&localDest;
+        }
+        const Normalizer2 *n2=Normalizer2Factory::getInstance(mode, status);
+        if(U_SUCCESS(status)) {
+            if(options&UNORM_UNICODE_3_2) {
+                FilteredNormalizer2(*n2, *uniset_getUnicode32Instance(status)).
+                    normalize(source, *dest, status);
+            } else {
+                n2->normalize(source, *dest, status);
+            }
+        }
+        if(dest==&localDest && U_SUCCESS(status)) {
+            result=*dest;
         }
     }
 }
 
-UNormalizationCheckResult
-Normalizer::quickCheck(const UnicodeString& source,
-                       UNormalizationMode mode, 
-                       UErrorCode &status) {
-    if(U_FAILURE(status)) {
-        return UNORM_MAYBE;
-    }
-
-    return unorm_quickCheck(source.getBuffer(), source.length(),
-                            mode, &status);
-}
-
-void
+void U_EXPORT2
 Normalizer::compose(const UnicodeString& source, 
                     UBool compat, int32_t options,
                     UnicodeString& result, 
                     UErrorCode &status) {
-    if(source.isBogus() || U_FAILURE(status)) {
-        result.setToBogus();
-        if(U_SUCCESS(status)) {
-            status=U_ILLEGAL_ARGUMENT_ERROR;
-        }
-    } else {
-        UChar *buffer=result.getBuffer(source.length());
-        int32_t length=unorm_compose(buffer, result.getCapacity(),
-                                     source.getBuffer(), source.length(),
-                                     compat, (options&IGNORE_HANGUL)!=0,
-                                     &status);
-        result.releaseBuffer(length);
-        if(status==U_BUFFER_OVERFLOW_ERROR) {
-            status=U_ZERO_ERROR;
-            buffer=result.getBuffer(length);
-            length=unorm_compose(buffer, result.getCapacity(),
-                                 source.getBuffer(), source.length(),
-                                 compat, (options&IGNORE_HANGUL)!=0,
-                                 &status);
-            result.releaseBuffer(length);
-        }
-
-        if(U_FAILURE(status)) {
-            result.setToBogus();
-        }
-    }
+    normalize(source, compat ? UNORM_NFKC : UNORM_NFC, options, result, status);
 }
 
-void
+void U_EXPORT2
 Normalizer::decompose(const UnicodeString& source, 
                       UBool compat, int32_t options,
                       UnicodeString& result, 
                       UErrorCode &status) {
-    if(source.isBogus() || U_FAILURE(status)) {
-        result.setToBogus();
-        if(U_SUCCESS(status)) {
-            status=U_ILLEGAL_ARGUMENT_ERROR;
+    normalize(source, compat ? UNORM_NFKD : UNORM_NFD, options, result, status);
+}
+
+UNormalizationCheckResult
+Normalizer::quickCheck(const UnicodeString& source,
+                       UNormalizationMode mode, int32_t options,
+                       UErrorCode &status) {
+    const Normalizer2 *n2=Normalizer2Factory::getInstance(mode, status);
+    if(U_SUCCESS(status)) {
+        if(options&UNORM_UNICODE_3_2) {
+            return FilteredNormalizer2(*n2, *uniset_getUnicode32Instance(status)).
+                quickCheck(source, status);
+        } else {
+            return n2->quickCheck(source, status);
         }
     } else {
-        UChar *buffer=result.getBuffer(source.length());
-        int32_t length=unorm_decompose(buffer, result.getCapacity(),
-                                     source.getBuffer(), source.length(),
-                                     compat, (options&IGNORE_HANGUL)!=0,
-                                     &status);
-        result.releaseBuffer(length);
-        if(status==U_BUFFER_OVERFLOW_ERROR) {
-            status=U_ZERO_ERROR;
-            buffer=result.getBuffer(length);
-            length=unorm_decompose(buffer, result.getCapacity(),
-                                   source.getBuffer(), source.length(),
-                                   compat, (options&IGNORE_HANGUL)!=0,
-                                   &status);
-            result.releaseBuffer(length);
-        }
-
-        if(U_FAILURE(status)) {
-            result.setToBogus();
-        }
+        return UNORM_MAYBE;
     }
 }
 
-UnicodeString &
+UBool
+Normalizer::isNormalized(const UnicodeString& source,
+                         UNormalizationMode mode, int32_t options,
+                         UErrorCode &status) {
+    const Normalizer2 *n2=Normalizer2Factory::getInstance(mode, status);
+    if(U_SUCCESS(status)) {
+        if(options&UNORM_UNICODE_3_2) {
+            return FilteredNormalizer2(*n2, *uniset_getUnicode32Instance(status)).
+                isNormalized(source, status);
+        } else {
+            return n2->isNormalized(source, status);
+        }
+    } else {
+        return FALSE;
+    }
+}
+
+UnicodeString & U_EXPORT2
 Normalizer::concatenate(UnicodeString &left, UnicodeString &right,
                         UnicodeString &result,
                         UNormalizationMode mode, int32_t options,
@@ -280,26 +213,27 @@ Normalizer::concatenate(UnicodeString &left, UnicodeString &right,
             errorCode=U_ILLEGAL_ARGUMENT_ERROR;
         }
     } else {
-        UChar *buffer=result.getBuffer(left.length()+right.length());
-        int32_t length=unorm_concatenate(left.getBuffer(), left.length(),
-                                         right.getBuffer(), right.length(),
-                                         buffer, result.getCapacity(),
-                                         mode, options,
-                                         &errorCode);
-        result.releaseBuffer(length);
-        if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
-            errorCode=U_ZERO_ERROR;
-            buffer=result.getBuffer(length);
-            int32_t length=unorm_concatenate(left.getBuffer(), left.length(),
-                                             right.getBuffer(), right.length(),
-                                             buffer, result.getCapacity(),
-                                             mode, options,
-                                             &errorCode);
-            result.releaseBuffer(length);
-        }
+        UnicodeString localDest;
+        UnicodeString *dest;
 
-        if(U_FAILURE(errorCode)) {
-            result.setToBogus();
+        if(&right!=&result) {
+            dest=&result;
+        } else {
+            // the right and result strings are the same object, use a temporary one
+            dest=&localDest;
+        }
+        *dest=left;
+        const Normalizer2 *n2=Normalizer2Factory::getInstance(mode, errorCode);
+        if(U_SUCCESS(errorCode)) {
+            if(options&UNORM_UNICODE_3_2) {
+                FilteredNormalizer2(*n2, *uniset_getUnicode32Instance(errorCode)).
+                    append(*dest, right, errorCode);
+            } else {
+                n2->append(*dest, right, errorCode);
+            }
+        }
+        if(dest==&localDest && U_SUCCESS(errorCode)) {
+            result=*dest;
         }
     }
     return result;
@@ -310,7 +244,7 @@ Normalizer::concatenate(UnicodeString &left, UnicodeString &right,
 //-------------------------------------------------------------------------
 
 /**
- * Return the current character in the normalized text->
+ * Return the current character in the normalized text.
  */
 UChar32 Normalizer::current() {
     if(bufferPos<buffer.length() || nextNormalize()) {
@@ -351,43 +285,20 @@ UChar32 Normalizer::previous() {
 }
 
 void Normalizer::reset() {
-    currentIndex=nextIndex=text->move(text, 0, UITER_START);
+    currentIndex=nextIndex=text->setToStart();
     clearBuffer();
 }
 
 void
 Normalizer::setIndexOnly(int32_t index) {
-    currentIndex=nextIndex=text->move(text, index, UITER_ZERO); // validates index
+    text->setIndex(index);  // pins index
+    currentIndex=nextIndex=text->getIndex();
     clearBuffer();
 }
 
 /**
- * Set the iteration position in the input text that is being normalized
- * and return the first normalized character at that position.
- * <p>
- * <b>Note:</b> This method sets the position in the <em>input</em> text,
- * while {@link #next} and {@link #previous} iterate through characters
- * in the normalized <em>output</em>.  This means that there is not
- * necessarily a one-to-one correspondence between characters returned
- * by <tt>next</tt> and <tt>previous</tt> and the indices passed to and
- * returned from <tt>setIndex</tt> and {@link #getIndex}.
- * <p>
- * @param index the desired index in the input text->
- *
- * @return      the first normalized character that is the result of iterating
- *              forward starting at the given index.
- *
- * @throws IllegalArgumentException if the given index is less than
- *          {@link #getBeginIndex} or greater than {@link #getEndIndex}.
- */
-UChar32 Normalizer::setIndex(int32_t index) {
-    setIndexOnly(index);
-    return current();
-}
-
-/**
- * Return the first character in the normalized text->  This resets
- * the <tt>Normalizer's</tt> position to the beginning of the text->
+ * Return the first character in the normalized text.  This resets
+ * the <tt>Normalizer's</tt> position to the beginning of the text.
  */
 UChar32 Normalizer::first() {
     reset();
@@ -395,12 +306,12 @@ UChar32 Normalizer::first() {
 }
 
 /**
- * Return the last character in the normalized text->  This resets
+ * Return the last character in the normalized text.  This resets
  * the <tt>Normalizer's</tt> position to be just before the
  * the input text corresponding to that normalized character.
  */
 UChar32 Normalizer::last() {
-    currentIndex=nextIndex=text->move(text, 0, UITER_LIMIT);
+    currentIndex=nextIndex=text->setToEnd();
     clearBuffer();
     return previous();
 }
@@ -428,21 +339,21 @@ int32_t Normalizer::getIndex() const {
 }
 
 /**
- * Retrieve the index of the start of the input text->  This is the begin index
+ * Retrieve the index of the start of the input text.  This is the begin index
  * of the <tt>CharacterIterator</tt> or the start (i.e. 0) of the <tt>String</tt>
  * over which this <tt>Normalizer</tt> is iterating
  */
 int32_t Normalizer::startIndex() const {
-    return text->getIndex(text, UITER_START);
+    return text->startIndex();
 }
 
 /**
- * Retrieve the index of the end of the input text->  This is the end index
+ * Retrieve the index of the end of the input text.  This is the end index
  * of the <tt>CharacterIterator</tt> or the length of the <tt>String</tt>
  * over which this <tt>Normalizer</tt> is iterating
  */
 int32_t Normalizer::endIndex() const {
-    return text->getIndex(text, UITER_LIMIT);
+    return text->endIndex();
 }
 
 //-------------------------------------------------------------------------
@@ -453,6 +364,7 @@ void
 Normalizer::setMode(UNormalizationMode newMode) 
 {
     fUMode = newMode;
+    init();
 }
 
 UNormalizationMode
@@ -470,6 +382,7 @@ Normalizer::setOption(int32_t option,
     } else {
         fOptions &= (~option);
     }
+    init();
 }
 
 UBool
@@ -480,7 +393,7 @@ Normalizer::getOption(int32_t option) const
 
 /**
  * Set the input text over which this <tt>Normalizer</tt> will iterate.
- * The iteration position is set to the beginning of the input text->
+ * The iteration position is set to the beginning of the input text.
  */
 void
 Normalizer::setText(const UnicodeString& newText, 
@@ -494,8 +407,8 @@ Normalizer::setText(const UnicodeString& newText,
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
-    delete (CharacterIterator *)(text->context);
-    text->context = newIter;
+    delete text;
+    text = newIter;
     reset();
 }
 
@@ -515,8 +428,8 @@ Normalizer::setText(const CharacterIterator& newText,
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
-    delete (CharacterIterator *)(text->context);
-    text->context = newIter;
+    delete text;
+    text = newIter;
     reset();
 }
 
@@ -533,8 +446,8 @@ Normalizer::setText(const UChar* newText,
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
-    delete (CharacterIterator *)(text->context);
-    text->context = newIter;
+    delete text;
+    text = newIter;
     reset();
 }
 
@@ -545,7 +458,7 @@ Normalizer::setText(const UChar* newText,
 void
 Normalizer::getText(UnicodeString&  result) 
 {
-    ((CharacterIterator *)(text->context))->getText(result);
+    text->getText(result);
 }
 
 //-------------------------------------------------------------------------
@@ -559,73 +472,51 @@ void Normalizer::clearBuffer() {
 
 UBool
 Normalizer::nextNormalize() {
-    UChar *p;
-    int32_t length;
-    UErrorCode errorCode;
-
     clearBuffer();
     currentIndex=nextIndex;
-    text->move(text, nextIndex, UITER_ZERO);
-    if(!text->hasNext(text)) {
+    text->setIndex(nextIndex);
+    if(!text->hasNext()) {
         return FALSE;
     }
-
-    errorCode=U_ZERO_ERROR;
-    p=buffer.getBuffer(-1);
-    length=unorm_next(text, p, buffer.getCapacity(),
-                      fUMode, fOptions!=0,
-                      TRUE, 0,
-                      &errorCode);
-    buffer.releaseBuffer(length);
-    if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
-        errorCode=U_ZERO_ERROR;
-        text->move(text, nextIndex, UITER_ZERO);
-        p=buffer.getBuffer(length);
-        length=unorm_next(text, p, buffer.getCapacity(),
-                          fUMode, fOptions!=0,
-                          TRUE, 0,
-                          &errorCode);
-        buffer.releaseBuffer(length);
+    // Skip at least one character so we make progress.
+    UnicodeString segment(text->next32PostInc());
+    while(text->hasNext()) {
+        UChar32 c;
+        if(fNorm2->hasBoundaryBefore(c=text->next32PostInc())) {
+            text->move32(-1, CharacterIterator::kCurrent);
+            break;
+        }
+        segment.append(c);
     }
-
-    nextIndex=text->getIndex(text, UITER_CURRENT);
+    nextIndex=text->getIndex();
+    UErrorCode errorCode=U_ZERO_ERROR;
+    fNorm2->normalize(segment, buffer, errorCode);
     return U_SUCCESS(errorCode) && !buffer.isEmpty();
 }
 
 UBool
 Normalizer::previousNormalize() {
-    UChar *p;
-    int32_t length;
-    UErrorCode errorCode;
-
     clearBuffer();
     nextIndex=currentIndex;
-    text->move(text, currentIndex, UITER_ZERO);
-    if(!text->hasPrevious(text)) {
+    text->setIndex(currentIndex);
+    if(!text->hasPrevious()) {
         return FALSE;
     }
-
-    errorCode=U_ZERO_ERROR;
-    p=buffer.getBuffer(-1);
-    length=unorm_previous(text, p, buffer.getCapacity(),
-                          fUMode, fOptions,
-                          TRUE, 0,
-                          &errorCode);
-    buffer.releaseBuffer(length);
-    if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
-        errorCode=U_ZERO_ERROR;
-        text->move(text, currentIndex, UITER_ZERO);
-        p=buffer.getBuffer(length);
-        length=unorm_previous(text, p, buffer.getCapacity(),
-                              fUMode, fOptions,
-                              TRUE, 0,
-                              &errorCode);
-        buffer.releaseBuffer(length);
+    UnicodeString segment;
+    while(text->hasPrevious()) {
+        UChar32 c=text->previous32();
+        segment.insert(0, c);
+        if(fNorm2->hasBoundaryBefore(c)) {
+            break;
+        }
     }
-
+    currentIndex=text->getIndex();
+    UErrorCode errorCode=U_ZERO_ERROR;
+    fNorm2->normalize(segment, buffer, errorCode);
     bufferPos=buffer.length();
-    currentIndex=text->getIndex(text, UITER_CURRENT);
     return U_SUCCESS(errorCode) && !buffer.isEmpty();
 }
 
 U_NAMESPACE_END
+
+#endif /* #if !UCONFIG_NO_NORMALIZATION */
