@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *                                                                            *
-* Copyright (C) 2001-2006, International Business Machines                   *
+* Copyright (C) 2001-2010, International Business Machines                   *
 *                Corporation and others. All Rights Reserved.                *
 *                                                                            *
 ******************************************************************************
@@ -25,6 +25,9 @@
 #include "ucln.h"
 #include "cmemory.h"
 #include "uassert.h"
+#include "unicode/icuplug.h"
+#include "icuplugimp.h"
+
 
 static UBool gICUInitialized = FALSE;
 static UMTX  gICUInitMutex   = NULL;
@@ -48,9 +51,9 @@ u_cleanup(void)
     cmemory_cleanup();       /* undo any heap functions set by u_setMemoryFunctions(). */
     gICUInitialized = FALSE;
     UTRACE_EXIT();           /* Must be before utrace_cleanup(), which turns off tracing. */
-#if U_ENABLE_TRACING
+/*#if U_ENABLE_TRACING*/
     utrace_cleanup();
-#endif
+/*#endif*/
 }
 
 /*
@@ -62,8 +65,9 @@ u_cleanup(void)
 U_CAPI void U_EXPORT2
 u_init(UErrorCode *status) {
     UTRACE_ENTRY_OC(UTRACE_U_INIT);
-    /* Make sure the global mutexes are initialized. */
-    umtx_init(NULL);
+    /* initialize plugins */
+    uplug_init(status);
+
     umtx_lock(&gICUInitMutex);
     if (gICUInitialized || U_FAILURE(*status)) {
         umtx_unlock(&gICUInitMutex);
@@ -85,7 +89,7 @@ u_init(UErrorCode *status) {
      * available.
      */
 #if !UCONFIG_NO_CONVERSION
-    ucnv_io_countTotalAliases(status);
+    ucnv_io_countKnownConverters(status);
 #endif
 #else
     /* Do any required init for services that don't have open operations
@@ -94,6 +98,11 @@ u_init(UErrorCode *status) {
      * initialization had occurred).
      */
 
+    /* TODO:  Completely remove this section.  u_init() is now publicly being 
+     *        advertised as never being required for thread safety, and we cannot
+     *        bring back this requirement.
+     */
+
     /* Char Properties */
     uprv_haveProperties(status);
 
@@ -106,6 +115,7 @@ u_init(UErrorCode *status) {
     unorm_haveData(status);
 #endif
 #endif
+
     gICUInitialized = TRUE;    /* TODO:  don't set if U_FAILURE? */
     umtx_unlock(&gICUInitMutex);
     UTRACE_EXIT_STATUS(*status);
