@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1999-2006, International Business Machines
+*   Copyright (C) 1999-2010, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -18,6 +18,7 @@
 #define __UDATA_H__
 
 #include "unicode/utypes.h"
+#include "unicode/localpointer.h"
 
 U_CDECL_BEGIN
 
@@ -220,7 +221,7 @@ udata_open(const char *path, const char *type, const char *name,
  * logically prepended to the ICU data directory string.</p>
  *
  * <p>For details about ICU data loading see the User Guide
- * Data Management chapter. (http://icu.sourceforge.net/userguide/icudata.html)</p>
+ * Data Management chapter. (http://icu-project.org/userguide/icudata.html)</p>
  *
  * @param path Specifies an absolute path and/or a basename for the
  *             finding of the data in the file system.
@@ -256,6 +257,25 @@ udata_openChoice(const char *path, const char *type, const char *name,
 U_STABLE void U_EXPORT2
 udata_close(UDataMemory *pData);
 
+#if U_SHOW_CPLUSPLUS_API
+
+U_NAMESPACE_BEGIN
+
+/**
+ * \class LocalUDataMemoryPointer
+ * "Smart pointer" class, closes a UDataMemory via udata_close().
+ * For most methods see the LocalPointerBase base class.
+ *
+ * @see LocalPointerBase
+ * @see LocalPointer
+ * @draft ICU 4.4
+ */
+U_DEFINE_LOCAL_OPEN_POINTER(LocalUDataMemoryPointer, UDataMemory, udata_close);
+
+U_NAMESPACE_END
+
+#endif
+
 /**
  * Get the pointer to the actual data inside the data memory.
  * The data is read-only.
@@ -298,20 +318,27 @@ udata_getInfo(UDataMemory *pData, UDataInfo *pInfo);
  * the data that has been loaded from a dll by the operating system,
  * as shown in this code:
  *
- *       extern const  char U_IMPORT U_ICUDATA_ENTRY_POINT []; 
+ *       extern const char U_IMPORT U_ICUDATA_ENTRY_POINT [];
  *        // U_ICUDATA_ENTRY_POINT is same as entry point specified to pkgdata tool
  *       UErrorCode  status = U_ZERO_ERROR;
  *
  *       udata_setCommonData(&U_ICUDATA_ENTRY_POINT, &status);
  *
- * Warning: ICU must NOT have even attempted to access its data yet
- * when this call is made, or U_USING_DEFAULT_WARNING code will
- * be returned. Be careful of UnicodeStrings in static initialization which
- * may attempt to load a converter (use the UNICODE_STRING(x) macro instead).
- *
- * Also note that it is important that the declaration be as above. The entry point
+ * It is important that the declaration be as above. The entry point
  * must not be declared as an extern void*.
  *
+ * Starting with ICU 4.4, it is possible to set several data packages,
+ * one per call to this function.
+ * udata_open() will look for data in the multiple data packages in the order
+ * in which they were set.
+ * The position of the linked-in or default-name ICU .data package in the
+ * search list depends on when the first data item is loaded that is not contained
+ * in the already explicitly set packages.
+ * If data was loaded implicitly before the first call to this function
+ * (for example, via opening a converter, constructing a UnicodeString
+ * from default-codepage data, using formatting or collation APIs, etc.),
+ * then the default data will be first in the list.
+ *
  * This function has no effect on application (non ICU) data.  See udata_setAppData()
  * for similar functionality for application data.
  *
@@ -319,7 +346,6 @@ udata_getInfo(UDataMemory *pData, UDataInfo *pInfo);
  * @param err outgoing error status <code>U_USING_DEFAULT_WARNING, U_UNSUPPORTED_ERROR</code>
  * @stable ICU 2.0
  */
-
 U_STABLE void U_EXPORT2
 udata_setCommonData(const void *data, UErrorCode *err);
 
@@ -354,18 +380,18 @@ udata_setAppData(const char *packageName, const void *data, UErrorCode *err);
 /**
  * Possible settings for udata_setFileAccess()
  * @see udata_setFileAccess
- * @draft ICU 3.4
+ * @stable ICU 3.4
  */
 typedef enum UDataFileAccess {
-    /** ICU does not access the file system for data loading. */
-    UDATA_NO_FILES,
+    /** ICU looks for data in single files first, then in packages. (default) */
+    UDATA_FILES_FIRST,
     /** ICU only loads data from packages, not from single files. */
     UDATA_ONLY_PACKAGES,
     /** ICU loads data from packages first, and only from single files
         if the data cannot be found in a package. */
     UDATA_PACKAGES_FIRST,
-    /** ICU looks for data in single files first, then in packages. (default) */
-    UDATA_FILES_FIRST,
+    /** ICU does not access the file system for data loading. */
+    UDATA_NO_FILES,
     /** An alias for the default access mode. */
     UDATA_DEFAULT_ACCESS = UDATA_FILES_FIRST,
     UDATA_FILE_ACCESS_COUNT
@@ -373,15 +399,15 @@ typedef enum UDataFileAccess {
 
 /**
  * This function may be called to control how ICU loads data. It must be called
- * before any ICU data is loaded, including application data loaded with ures/ResourceBundle or
- * udata APIs. It should be called before u_init.  This function is not multithread safe.  
+ * before any ICU data is loaded, including application data loaded with 
+ * ures/ResourceBundle or udata APIs. This function is not multithread safe.  
  * The results of calling it while other threads are loading data are undefined.
  * @param access The type of file access to be used
  * @param status Error code.
  * @see UDataFileAccess
- * @draft ICU 3.4 
+ * @stable ICU 3.4 
  */
-U_DRAFT void U_EXPORT2
+U_STABLE void U_EXPORT2
 udata_setFileAccess(UDataFileAccess access, UErrorCode *status);
 
 U_CDECL_END
