@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-*   Copyright (C) 1997-2001, International Business Machines
+*   Copyright (C) 1997-2009, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 *   Date        Name        Description
@@ -12,8 +12,13 @@
 #define UTRANS_H
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
+#include "unicode/localpointer.h"
 #include "unicode/urep.h"
 #include "unicode/parseerr.h"
+#include "unicode/uenum.h"
 
 /********************************************************************
  * General Notes
@@ -28,12 +33,24 @@
  * transliteration services.  Transliteration services are implemented
  * as C++ classes.  The comments and documentation in this header
  * assume the reader is familiar with the C++ headers translit.h and
- * rbt.h and associated documentation.
+ * associated documentation.
  *
  * A significant but incomplete subset of the C++ transliteration
  * services are available to C code through this header.  In order to
  * access more complex transliteration services, refer to the C++
  * headers and documentation.
+ *
+ * There are two sets of functions for working with transliterator IDs:
+ *
+ * An old, deprecated set uses char * IDs, which works for true and pure
+ * identifiers that these APIs were designed for,
+ * for example "Cyrillic-Latin".
+ * It does not work when the ID contains filters ("[:Script=Cyrl:]")
+ * or even a complete set of rules because then the ID string contains more
+ * than just "invariant" characters (see utypes.h).
+ *
+ * A new set of functions replaces the old ones and uses UChar * IDs,
+ * paralleling the UnicodeString IDs in the C++ API. (New in ICU 2.8.)
  */
 
 /********************************************************************
@@ -45,7 +62,7 @@
  * and close with utrans_close() when done.  Equivalent to the C++ class
  * Transliterator and its subclasses.
  * @see Transliterator
- * @stable
+ * @stable ICU 2.0
  */
 typedef void* UTransliterator;
 
@@ -55,21 +72,21 @@ typedef void* UTransliterator;
  * Specified when a transliterator is opened.  An "A-B" transliterator
  * transliterates A to B when operating in the forward direction, and
  * B to A when operating in the reverse direction.
- * @stable
+ * @stable ICU 2.0
  */
-typedef enum _UTransDirection {
+typedef enum UTransDirection {
     
     /**
-     * UTRANS_FORWARD means from <source> to <target> for a
-     * transliterator with ID <source>-<target>.  For a transliterator
+     * UTRANS_FORWARD means from &lt;source&gt; to &lt;target&gt; for a
+     * transliterator with ID &lt;source&gt;-&lt;target&gt;.  For a transliterator
      * opened using a rule, it means forward direction rules, e.g.,
      * "A > B".
      */
     UTRANS_FORWARD,
 
     /**
-     * UTRANS_REVERSE means from <target> to <source> for a
-     * transliterator with ID <source>-<target>.  For a transliterator
+     * UTRANS_REVERSE means from &lt;target&gt; to &lt;source&gt; for a
+     * transliterator with ID &lt;source&gt;-&lt;target&gt;.  For a transliterator
      * opened using a rule, it means reverse direction rules, e.g.,
      * "A < B".
      */
@@ -97,9 +114,9 @@ typedef enum _UTransDirection {
  * of a surrogate pair.  If any index does split a surrogate pair,
  * results are unspecified.
  *
- * @stable
+ * @stable ICU 2.0
  */
-typedef struct _UTransPosition {
+typedef struct UTransPosition {
 
     /**
      * Beginning index, inclusive, of the context to be considered for
@@ -107,6 +124,7 @@ typedef struct _UTransPosition {
      * anything before this index.  INPUT/OUTPUT parameter: This parameter
      * is updated by a transliteration operation to reflect the maximum
      * amount of antecontext needed by a transliterator.
+     * @stable ICU 2.4
      */
     int32_t contextStart;
     
@@ -116,6 +134,7 @@ typedef struct _UTransPosition {
      * anything at or after this index.  INPUT/OUTPUT parameter: This
      * parameter is updated to reflect changes in the length of the
      * text, but points to the same logical position in the text.
+     * @stable ICU 2.4
      */
     int32_t contextLimit;
     
@@ -124,6 +143,7 @@ typedef struct _UTransPosition {
      * INPUT/OUTPUT parameter: This parameter is advanced past
      * characters that have already been transliterated by a
      * transliteration operation.
+     * @stable ICU 2.4
      */
     int32_t start;
     
@@ -132,6 +152,7 @@ typedef struct _UTransPosition {
      * INPUT/OUTPUT parameter: This parameter is updated to reflect
      * changes in the length of the text, but points to the same
      * logical position in the text.
+     * @stable ICU 2.4
      */
     int32_t limit;
 
@@ -148,29 +169,30 @@ typedef struct _UTransPosition {
  * Any non-NULL result from this function should later be closed with
  * utrans_close().
  *
- * @param id a valid ID, as returned by utrans_getAvailableID()
+ * @param id a valid transliterator ID
+ * @param idLength the length of the ID string, or -1 if NUL-terminated
  * @param dir the desired direction
- * @param rules the transliterator rules.  See the C++ header rbt.h
- * for rules syntax. If NULL then a system transliterator matching 
- * the ID is returned.
+ * @param rules the transliterator rules.  See the C++ header rbt.h for
+ *              rules syntax. If NULL then a system transliterator matching
+ *              the ID is returned.
  * @param rulesLength the length of the rules, or -1 if the rules
- * are zero-terminated.
- * @param dir the desired direction
- * @param parseErr a pointer to a UParseError struct to receive the
- * details of any parsing errors. This parameter may be NULL if no
- * parsing error details are desired.
- * @param status a pointer to the UErrorCode
+ *                    are NUL-terminated.
+ * @param parseError a pointer to a UParseError struct to receive the details
+ *                   of any parsing errors. This parameter may be NULL if no
+ *                   parsing error details are desired.
+ * @param pErrorCode a pointer to the UErrorCode
  * @return a transliterator pointer that may be passed to other
- * utrans_xxx() functions, or NULL if the open call fails.
- * @draft ICU 2.0
+ *         utrans_xxx() functions, or NULL if the open call fails.
+ * @stable ICU 2.8
  */
-U_CAPI UTransliterator* U_EXPORT2 
-utrans_open(const char* id,
-            UTransDirection dir,
-            const UChar* rules,         /* may be Null */
-            int32_t rulesLength,        /* -1 if null-terminated */ 
-            UParseError* parseError,    /* may be Null */
-            UErrorCode* status);
+U_STABLE UTransliterator* U_EXPORT2
+utrans_openU(const UChar *id,
+             int32_t idLength,
+             UTransDirection dir,
+             const UChar *rules,
+             int32_t rulesLength,
+             UParseError *parseError,
+             UErrorCode *pErrorCode);
 
 /**
  * Open an inverse of an existing transliterator.  For this to work,
@@ -185,9 +207,9 @@ utrans_open(const char* id,
  * @param status a pointer to the UErrorCode
  * @return a pointer to a newly-opened transliterator that is the
  * inverse of trans, or NULL if the open call fails.
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI UTransliterator* U_EXPORT2 
+U_STABLE UTransliterator* U_EXPORT2 
 utrans_openInverse(const UTransliterator* trans,
                    UErrorCode* status);
 
@@ -199,38 +221,57 @@ utrans_openInverse(const UTransliterator* trans,
  * @param status a pointer to the UErrorCode
  * @return a transliterator pointer that may be passed to other
  * utrans_xxx() functions, or NULL if the clone call fails.
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI UTransliterator* U_EXPORT2 
+U_STABLE UTransliterator* U_EXPORT2 
 utrans_clone(const UTransliterator* trans,
              UErrorCode* status);
 
 /**
  * Close a transliterator.  Any non-NULL pointer returned by
  * utrans_openXxx() or utrans_clone() should eventually be closed.
- * @stable
+ * @param trans the transliterator to be closed.
+ * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_close(UTransliterator* trans);
 
+#if U_SHOW_CPLUSPLUS_API
+
+U_NAMESPACE_BEGIN
+
+/**
+ * \class LocalUTransliteratorPointer
+ * "Smart pointer" class, closes a UTransliterator via utrans_close().
+ * For most methods see the LocalPointerBase base class.
+ *
+ * @see LocalPointerBase
+ * @see LocalPointer
+ * @draft ICU 4.4
+ */
+U_DEFINE_LOCAL_OPEN_POINTER(LocalUTransliteratorPointer, UTransliterator, utrans_close);
+
+U_NAMESPACE_END
+
+#endif
+
 /**
  * Return the programmatic identifier for this transliterator.
- * If this identifier is passed to utrans_open(), it will open
+ * If this identifier is passed to utrans_openU(), it will open
  * a transliterator equivalent to this one, if the ID has been
  * registered.
+ *
  * @param trans the transliterator to return the ID of.
- * @param buf the buffer in which to receive the ID.  This may be
- * NULL, in which case no characters are copied.
- * @param bufCapacity the capacity of the buffer.  Ignored if buf is
- * NULL.
- * @return the actual length of the ID, not including
- * zero-termination.  This may be greater than bufCapacity.
- * @stable
+ * @param resultLength pointer to an output variable receiving the length
+ *        of the ID string; can be NULL
+ * @return the NUL-terminated ID string. This pointer remains
+ * valid until utrans_close() is called on this transliterator.
+ *
+ * @stable ICU 2.8
  */
-U_CAPI int32_t U_EXPORT2 
-utrans_getID(const UTransliterator* trans,
-             char* buf,
-             int32_t bufCapacity);
+U_STABLE const UChar * U_EXPORT2
+utrans_getUnicodeID(const UTransliterator *trans,
+                    int32_t *resultLength);
 
 /**
  * Register an open transliterator with the system.  When
@@ -241,25 +282,26 @@ utrans_getID(const UTransliterator* trans,
  * <p>NOTE: After this call the system owns the adoptedTrans and will
  * close it.  The user must not call utrans_close() on adoptedTrans.
  *
- * @param trans a transliterator, typically the result of
+ * @param adoptedTrans a transliterator, typically the result of
  * utrans_openRules(), to be registered with the system.
  * @param status a pointer to the UErrorCode
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_register(UTransliterator* adoptedTrans,
                 UErrorCode* status);
 
 /**
  * Unregister a transliterator from the system.  After this call the
  * system will no longer recognize the given ID when passed to
- * utrans_open().  If the id is invalid then nothing is done.
+ * utrans_open(). If the ID is invalid then nothing is done.
  *
- * @param id a zero-terminated ID
- * @stable
+ * @param id an ID to unregister
+ * @param idLength the length of id, or -1 if id is zero-terminated
+ * @stable ICU 2.8
  */
-U_CAPI void U_EXPORT2 
-utrans_unregister(const char* id);
+U_STABLE void U_EXPORT2
+utrans_unregisterID(const UChar* id, int32_t idLength);
 
 /**
  * Set the filter used by a transliterator.  A filter can be used to
@@ -277,9 +319,9 @@ utrans_unregister(const char* id);
  * filterPattern is zero-terminated
  * @param status a pointer to the UErrorCode
  * @see UnicodeSet
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_setFilter(UTransliterator* trans,
                  const UChar* filterPattern,
                  int32_t filterPatternLen,
@@ -287,32 +329,25 @@ utrans_setFilter(UTransliterator* trans,
 
 /**
  * Return the number of system transliterators.
- * @stable
+ * It is recommended to use utrans_openIDs() instead.
+ *
+ * @return the number of system transliterators.
+ * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_STABLE int32_t U_EXPORT2 
 utrans_countAvailableIDs(void);
 
 /**
- * Return the ID of the index-th system transliterator.  The result
- * is placed in the given buffer.  If the given buffer is too small,
- * the initial substring is copied to buf.  The result in buf is
- * always zero-terminated.
+ * Return a UEnumeration for the available transliterators.
  *
- * @param index the number of the transliterator to return.  Must
- * satisfy 0 <= index < utrans_countAvailableIDs().  If index is out
- * of range then it is treated as if it were 0.
- * @param buf the buffer in which to receive the ID.  This may be
- * NULL, in which case no characters are copied.
- * @param bufCapacity the capacity of the buffer.  Ignored if buf is
- * NULL.
- * @return the actual length of the index-th ID, not including
- * zero-termination.  This may be greater than bufCapacity.
- * @stable
+ * @param pErrorCode Pointer to the UErrorCode in/out parameter.
+ * @return UEnumeration for the available transliterators.
+ *         Close with uenum_close().
+ *
+ * @stable ICU 2.8
  */
-U_CAPI int32_t U_EXPORT2 
-utrans_getAvailableID(int32_t index,
-                      char* buf,
-                      int32_t bufCapacity);
+U_STABLE UEnumeration * U_EXPORT2
+utrans_openIDs(UErrorCode *pErrorCode);
 
 /********************************************************************
  * Transliteration API
@@ -339,9 +374,9 @@ utrans_getAvailableID(int32_t index,
  * </code><em>new-limit</em><code>)</code>, where <em>new-limit</em>
  * is the return value.
  * @param status a pointer to the UErrorCode
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_trans(const UTransliterator* trans,
              UReplaceable* rep,
              UReplaceableCallbacks* repFunc,
@@ -390,9 +425,9 @@ utrans_trans(const UTransliterator* trans,
  * @param pos a struct containing the start and limit indices of the
  * text to be read and the text to be transliterated
  * @param status a pointer to the UErrorCode
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_transIncremental(const UTransliterator* trans,
                         UReplaceable* rep,
                         UReplaceableCallbacks* repFunc,
@@ -428,9 +463,9 @@ utrans_transIncremental(const UTransliterator* trans,
  * </code><em>new-limit</em><code>)</code>, where <em>new-limit</em>
  * is the return value.
  * @param status a pointer to the UErrorCode
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_transUChars(const UTransliterator* trans,
                    UChar* text,
                    int32_t* textLength,
@@ -463,9 +498,9 @@ utrans_transUChars(const UTransliterator* trans,
  * text to be read and the text to be transliterated
  * @param status a pointer to the UErrorCode
  * @see utrans_transIncremental
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_transIncrementalUChars(const UTransliterator* trans,
                               UChar* text,
                               int32_t* textLength,
@@ -473,31 +508,96 @@ utrans_transIncrementalUChars(const UTransliterator* trans,
                               UTransPosition* pos,
                               UErrorCode* status);
 
+/* deprecated API ----------------------------------------------------------- */
+
+/* see utrans.h documentation for why these functions are deprecated */
 
-/********************* Deprecated API ************************************/
 /**
- *@deprecated Remove after Aug 2002
+ * Deprecated, use utrans_openU() instead.
+ * Open a custom transliterator, given a custom rules string 
+ * OR 
+ * a system transliterator, given its ID.  
+ * Any non-NULL result from this function should later be closed with
+ * utrans_close().
+ *
+ * @param id a valid ID, as returned by utrans_getAvailableID()
+ * @param dir the desired direction
+ * @param rules the transliterator rules.  See the C++ header rbt.h
+ * for rules syntax. If NULL then a system transliterator matching 
+ * the ID is returned.
+ * @param rulesLength the length of the rules, or -1 if the rules
+ * are zero-terminated.
+ * @param parseError a pointer to a UParseError struct to receive the
+ * details of any parsing errors. This parameter may be NULL if no
+ * parsing error details are desired.
+ * @param status a pointer to the UErrorCode
+ * @return a transliterator pointer that may be passed to other
+ * utrans_xxx() functions, or NULL if the open call fails.
+ * @deprecated ICU 2.8 Use utrans_openU() instead, see utrans.h
  */
+U_DEPRECATED UTransliterator* U_EXPORT2 
+utrans_open(const char* id,
+            UTransDirection dir,
+            const UChar* rules,         /* may be Null */
+            int32_t rulesLength,        /* -1 if null-terminated */ 
+            UParseError* parseError,    /* may be Null */
+            UErrorCode* status);
 
-#ifdef U_USE_DEPRECATED_FORMAT_API
+/**
+ * Deprecated, use utrans_getUnicodeID() instead.
+ * Return the programmatic identifier for this transliterator.
+ * If this identifier is passed to utrans_open(), it will open
+ * a transliterator equivalent to this one, if the ID has been
+ * registered.
+ * @param trans the transliterator to return the ID of.
+ * @param buf the buffer in which to receive the ID.  This may be
+ * NULL, in which case no characters are copied.
+ * @param bufCapacity the capacity of the buffer.  Ignored if buf is
+ * NULL.
+ * @return the actual length of the ID, not including
+ * zero-termination.  This may be greater than bufCapacity.
+ * @deprecated ICU 2.8 Use utrans_getUnicodeID() instead, see utrans.h
+ */
+U_DEPRECATED int32_t U_EXPORT2 
+utrans_getID(const UTransliterator* trans,
+             char* buf,
+             int32_t bufCapacity);
 
-#if ((U_ICU_VERSION_MAJOR_NUM != 2) || (U_ICU_VERSION_MINOR_NUM !=1))
-#   error "ICU version has changed. Please redefine the macros under U_USE_DEPRECATED_FORMAT_API pre-processor definition"
-#else 
-    U_CAPI UTransliterator* U_EXPORT2 
-    utrans_openRules(const char* id,
-                     const UChar* rules,
-                     int32_t rulesLength, /* -1 if null-terminated */
-                     UTransDirection dir,
-                     UParseError* parseErr, /* may be NULL */
-                     UErrorCode* status){
-        return utrans_open(id,dir,rules,rulesLength,parseErr,status);
-    }
+/**
+ * Deprecated, use utrans_unregisterID() instead.
+ * Unregister a transliterator from the system.  After this call the
+ * system will no longer recognize the given ID when passed to
+ * utrans_open().  If the id is invalid then nothing is done.
+ *
+ * @param id a zero-terminated ID
+ * @deprecated ICU 2.8 Use utrans_unregisterID() instead, see utrans.h
+ */
+U_DEPRECATED void U_EXPORT2 
+utrans_unregister(const char* id);
 
-#   define utrans_open_2_1(id,dir,status) utrans_open(id,dir,NULL,0,NULL,status)
+/**
+ * Deprecated, use utrans_openIDs() instead.
+ * Return the ID of the index-th system transliterator.  The result
+ * is placed in the given buffer.  If the given buffer is too small,
+ * the initial substring is copied to buf.  The result in buf is
+ * always zero-terminated.
+ *
+ * @param index the number of the transliterator to return.  Must
+ * satisfy 0 <= index < utrans_countAvailableIDs().  If index is out
+ * of range then it is treated as if it were 0.
+ * @param buf the buffer in which to receive the ID.  This may be
+ * NULL, in which case no characters are copied.
+ * @param bufCapacity the capacity of the buffer.  Ignored if buf is
+ * NULL.
+ * @return the actual length of the index-th ID, not including
+ * zero-termination.  This may be greater than bufCapacity.
+ * @deprecated ICU 2.8 Use utrans_openIDs() instead, see utrans.h
+ */
+U_DEPRECATED int32_t U_EXPORT2 
+utrans_getAvailableID(int32_t index,
+                      char* buf,
+                      int32_t bufCapacity);
 
-#endif
-#endif
-/********************* End **********************************************/
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
 
 #endif
