@@ -1,17 +1,15 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2006, International Business Machines Corporation and
+ * Copyright (c) 1997-2010, International Business Machines Corporation and
  * others. All Rights Reserved.
+ ********************************************************************
+ *
+ * File CMSGTST.C
+ *
+ * Modification History:
+ *        Name                     Description
+ *     Madhu Katragadda              Creation
  ********************************************************************/
-/********************************************************************************
-*
-* File CMSGTST.C
-*
-* Modification History:
-*        Name                     Description
-*     Madhu Katragadda              Creation
-*********************************************************************************
-*/
 /* C API TEST FOR MESSAGE FORMAT */
 
 #include "unicode/utypes.h"
@@ -42,7 +40,7 @@ static const char* const txt_testResultStrings[] = {
     "Quotes ', {, a 1 {0}",
     "Quotes ', {, a 1 {0}",
     "You deposited 1 times an amount of $3,456.00 on 1/12/70",
-    "{2,time,full}, for 3,456, 1 is 5:46:40 AM PT and full date is Monday, January 12, 1970",
+    "{2,time,full}, for 3,456, 1 is 5:46:40 AM Pacific Standard Time and full date is Monday, January 12, 1970",
     "{1,number,percent} for 1 is 345,600%"
 };
 
@@ -130,7 +128,8 @@ static void MessageFormatTest( void )
                 &status, 1, 3456.00, d1);
         }
         if(U_FAILURE(status)){
-            log_err("ERROR: failure in message format on testcase %d:  %s\n", i, myErrorName(status) );
+            log_data_err("ERROR: failure in message format on testcase %d:  %s (Are you missing data?)\n", i, myErrorName(status) );
+            continue;
         }
         if(u_strcmp(result, testResultStrings[i])==0){
             log_verbose("PASS: MessagFormat successful on testcase : %d\n", i);
@@ -161,7 +160,7 @@ static void MessageFormatTest( void )
                     &status, 1, 3456.00, d1);
             }
             if(U_FAILURE(status)){
-                log_err("ERROR: failure in message format on testcase %d:  %s\n", i, myErrorName(status) );
+                log_data_err("ERROR: failure in message format on testcase %d:  %s (Are you missing data?)\n", i, myErrorName(status) );
                 continue;
             }
             if(u_strcmp(result, testResultStrings[i])==0){
@@ -182,7 +181,7 @@ static void MessageFormatTest( void )
         UMessageFormat formatter = umsg_open(testCasePatterns[0],patternLength,"en_US",NULL,&ec);
 
         if(U_FAILURE(ec)){
-            log_err("umsg_open() failed for testCasePattens[%d].\n",i);
+            log_data_err("umsg_open() failed for testCasePattens[%d]. -> %s (Are you missing data?)\n",i, u_errorName(ec));
             return;
         }
         for(i = 0;i<cnt_testCases; i++){
@@ -285,9 +284,9 @@ static void TestSampleMessageFormat(void)
         u_formatMessage( "en_US", pattern, u_strlen(pattern), result, resultlength, &status, str, d);
     }
     if(U_FAILURE(status)){
-        log_err("Error: failure in message format on test#1: %s\n", myErrorName(status));
+        log_data_err("Error: failure in message format on test#1: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(u_strcmp(result, expected)==0)
+    else if(u_strcmp(result, expected)==0)
         log_verbose("PASS: MessagFormat successful on test#1\n");
     else{
         log_err("FAIL: Error in MessageFormat on test#1 \n GOT: %s EXPECTED: %s\n", 
@@ -316,9 +315,9 @@ static void TestSampleMessageFormat(void)
         u_formatMessage( "en_US", pattern, u_strlen(pattern), result, resultlength, &status, str, 23);
     }
     if(U_FAILURE(status)){
-        log_err("Error: failure in message format on test#2 : %s\n", myErrorName(status));
+        log_data_err("Error: failure in message format on test#2 : %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(u_strcmp(result, expected)==0)
+    else if(u_strcmp(result, expected)==0)
         log_verbose("PASS: MessagFormat successful on test#2\n");
     else{
         log_err("FAIL: Error in MessageFormat on test#2\n GOT: %s EXPECTED: %s\n", 
@@ -341,9 +340,9 @@ static void TestSampleMessageFormat(void)
         u_formatMessage( "en_US", pattern, u_strlen(pattern), result, resultlength, &status, str, 500.00);
     }
     if(U_FAILURE(status)){
-        log_err("Error: failure in message format on test#3 : %s\n", myErrorName(status));
+        log_data_err("Error: failure in message format on test#3 : %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(u_strcmp(result, expected)==0)
+    else if(u_strcmp(result, expected)==0)
         log_verbose("PASS: MessagFormat successful on test#3\n");
     else{
         log_err("FAIL: Error in MessageFormat on test#3\n GOT: %s EXPECTED %s\n", austrdup(result), 
@@ -360,7 +359,7 @@ static void TestSampleMessageFormat(void)
 static void TestNewFormatAndParseAPI(void)
 {
 
-    UChar *result, *tzID, *str;
+    UChar *result, tzID[4], str[25];
     UChar pattern[100];
     UChar expected[100];
     int32_t resultLengthOut, resultlength;
@@ -378,13 +377,11 @@ static void TestNewFormatAndParseAPI(void)
 
     log_verbose("Testing format and parse with parse error\n");
 
-    str=(UChar*)malloc(sizeof(UChar) * 25);
     u_uastrcpy(str, "disturbance in force");
-    tzID=(UChar*)malloc(sizeof(UChar) * 4);
     u_uastrcpy(tzID, "PST");
     cal=ucal_open(tzID, u_strlen(tzID), "en_US", UCAL_TRADITIONAL, &status);
     if(U_FAILURE(status)){
-        log_err("error in ucal_open caldef : %s\n", myErrorName(status) );
+        log_data_err("error in ucal_open caldef : %s - (Are you missing data?)\n", myErrorName(status) );
         return;
     }
     ucal_setDateTime(cal, 1999, UCAL_MARCH, 18, 0, 0, 0, &status);
@@ -400,7 +397,7 @@ static void TestNewFormatAndParseAPI(void)
     resultlength=1;
     fmt = umsg_open(pattern,u_strlen(pattern),"en_US",&parseError,&status);
     if(U_FAILURE(status)){
-        log_err("error in umsg_open  : %s\n", u_errorName(status) );
+        log_data_err("error in umsg_open  : %s (Are you missing data?)\n", u_errorName(status) );
         return;
     }
     result=(UChar*)malloc(sizeof(UChar) * resultlength);
@@ -455,8 +452,6 @@ static void TestNewFormatAndParseAPI(void)
     ucal_close(cal);
 
     free(result);
-    free(str);
-    free(tzID);
 
     ctest_resetTimeZone();
 }
@@ -488,12 +483,12 @@ static void TestSampleFormatAndParseWithError(void)
     u_uastrcpy(tzID, "PST");
     cal=ucal_open(tzID, u_strlen(tzID), "en_US", UCAL_TRADITIONAL, &status);
     if(U_FAILURE(status)){
-        log_err("error in ucal_open caldef : %s\n", myErrorName(status) );
+        log_data_err("error in ucal_open caldef : %s - (Are you missing data?)\n", myErrorName(status) );
     }
     ucal_setDateTime(cal, 1999, UCAL_MARCH, 18, 0, 0, 0, &status);
     d1=ucal_getMillis(cal, &status);
     if(U_FAILURE(status)){
-            log_err("Error: failure in get millis: %s\n", myErrorName(status) );
+            log_data_err("Error: failure in get millis: %s - (Are you missing data?)\n", myErrorName(status) );
     }
     
     log_verbose("\nTesting with pattern test#4");
@@ -511,9 +506,9 @@ static void TestSampleFormatAndParseWithError(void)
         
     }
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in message format test#4: %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in message format test#4: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(u_strcmp(result, expected)==0)
+    else if(u_strcmp(result, expected)==0)
         log_verbose("PASS: MessagFormat successful on test#4\n");
     else{
         log_err("FAIL: Error in MessageFormat on test#4\n GOT: %s EXPECTED: %s\n", austrdup(result),
@@ -526,9 +521,9 @@ static void TestSampleFormatAndParseWithError(void)
 
     u_parseMessageWithError("en_US", pattern, u_strlen(pattern), result, u_strlen(result), &parseError,&status, &d, ret, &value);
     if(U_FAILURE(status)){
-        log_err("ERROR: error in parsing: test#5: %s\n", myErrorName(status));
+        log_data_err("ERROR: error in parsing: test#5: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(value!=7 && u_strcmp(str,ret)!=0)
+    else if(value!=7 && u_strcmp(str,ret)!=0)
         log_err("FAIL: Error in parseMessage on test#5 \n");
     else
         log_verbose("PASS: parseMessage successful on test#5\n");
@@ -536,7 +531,7 @@ static void TestSampleFormatAndParseWithError(void)
     def1 = udat_open(UDAT_DEFAULT,UDAT_DEFAULT ,NULL, NULL, 0, NULL,0,&status);
     if(U_FAILURE(status))
     {
-        log_err("error in creating the dateformat using short date and time style:\n %s\n", myErrorName(status));
+        log_data_err("error in creating the dateformat using short date and time style: %s (Are you missing data?)\n", myErrorName(status));
     }else{
 
         if(u_strcmp(myDateFormat(def1, d), myDateFormat(def1, d1))==0)
@@ -581,12 +576,12 @@ static void TestSampleFormatAndParse(void)
     u_uastrcpy(tzID, "PST");
     cal=ucal_open(tzID, u_strlen(tzID), "en_US", UCAL_TRADITIONAL, &status);
     if(U_FAILURE(status)){
-        log_err("error in ucal_open caldef : %s\n", myErrorName(status) );
+        log_data_err("error in ucal_open caldef : %s - (Are you missing data?)\n", myErrorName(status) );
     }
     ucal_setDateTime(cal, 1999, UCAL_MARCH, 18, 0, 0, 0, &status);
     d1=ucal_getMillis(cal, &status);
     if(U_FAILURE(status)){
-            log_err("Error: failure in get millis: %s\n", myErrorName(status) );
+            log_data_err("Error: failure in get millis: %s - (Are you missing data?)\n", myErrorName(status) );
     }
     
     log_verbose("\nTesting with pattern test#4");
@@ -604,9 +599,9 @@ static void TestSampleFormatAndParse(void)
         
     }
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in message format test#4: %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in message format test#4: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(u_strcmp(result, expected)==0)
+    else if(u_strcmp(result, expected)==0)
         log_verbose("PASS: MessagFormat successful on test#4\n");
     else{
         log_err("FAIL: Error in MessageFormat on test#4\n GOT: %s EXPECTED: %s\n", austrdup(result),
@@ -619,9 +614,9 @@ static void TestSampleFormatAndParse(void)
 
     u_parseMessage("en_US", pattern, u_strlen(pattern), result, u_strlen(result), &status, &d, ret, &value);
     if(U_FAILURE(status)){
-        log_err("ERROR: error in parsing: test#5: %s\n", myErrorName(status));
+        log_data_err("ERROR: error in parsing: test#5: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(value!=7 && u_strcmp(str,ret)!=0)
+    else if(value!=7 && u_strcmp(str,ret)!=0)
         log_err("FAIL: Error in parseMessage on test#5 \n");
     else
         log_verbose("PASS: parseMessage successful on test#5\n");
@@ -629,7 +624,7 @@ static void TestSampleFormatAndParse(void)
     def1 = udat_open(UDAT_DEFAULT,UDAT_DEFAULT ,NULL, NULL, 0, NULL,0,&status);
     if(U_FAILURE(status))
     {
-        log_err("error in creating the dateformat using short date and time style:\n %s\n", myErrorName(status));
+        log_data_err("error in creating the dateformat using short date and time style: %s (Are you missing data?)\n", myErrorName(status));
     }else{
 
         if(u_strcmp(myDateFormat(def1, d), myDateFormat(def1, d1))==0)
@@ -649,6 +644,77 @@ static void TestSampleFormatAndParse(void)
     ctest_resetTimeZone();
 }
 
+/* Test message format with a Select option */
+static void TestMsgFormatSelect(void)
+{
+    UChar* str;
+    UChar* str1;
+    UErrorCode status = U_ZERO_ERROR;
+    UChar *result;
+    UChar pattern[100];
+    UChar expected[100];
+    int32_t resultlength,resultLengthOut;
+
+    str=(UChar*)malloc(sizeof(UChar) * 25);
+    u_uastrcpy(str, "Kirti");
+    str1=(UChar*)malloc(sizeof(UChar) * 25);
+    u_uastrcpy(str1, "female");
+    log_verbose("Testing message format with Select test #1\n:");
+    u_uastrcpy(pattern, "{0} est {1, select, female {all\\u00E9e} other {all\\u00E9}} \\u00E0 Paris.");
+    u_uastrcpy(expected, "Kirti est all\\u00E9e \\u00E0 Paris.");
+    resultlength=0;
+    resultLengthOut=u_formatMessage( "fr", pattern, u_strlen(pattern), NULL, resultlength, &status, str , str1);
+    if(status==U_BUFFER_OVERFLOW_ERROR)
+    {
+        status=U_ZERO_ERROR;
+        resultlength=resultLengthOut+1;
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
+        u_formatMessage( "fr", pattern, u_strlen(pattern), result, resultlength, &status, str , str1);
+        if(u_strcmp(result, expected)==0)
+            log_verbose("PASS: MessagFormat successful on Select test#1\n");
+        else{
+            log_err("FAIL: Error in MessageFormat on Select test#1\n GOT %s EXPECTED %s\n", austrdup(result),
+                austrdup(expected) );
+        }
+        free(result);
+    }
+    if(U_FAILURE(status)){
+        log_data_err("ERROR: failure in message format on Select test#1 : %s \n", myErrorName(status));
+    }
+    free(str);
+    free(str1);
+
+    /*Test a nested pattern*/
+    str=(UChar*)malloc(sizeof(UChar) * 25);
+    u_uastrcpy(str, "Noname");
+    str1=(UChar*)malloc(sizeof(UChar) * 25);
+    u_uastrcpy(str1, "other");
+    log_verbose("Testing message format with Select test #2\n:");
+    u_uastrcpy(pattern, "{0} est {1, select, female {{2,number,integer} all\\u00E9e} other {all\\u00E9}} \\u00E0 Paris.");
+    u_uastrcpy(expected, "Noname est all\\u00E9 \\u00E0 Paris.");
+    resultlength=0;
+    resultLengthOut=u_formatMessage( "fr", pattern, u_strlen(pattern), NULL, resultlength, &status, str , str1,6);
+    if(status==U_BUFFER_OVERFLOW_ERROR)
+    {
+        status=U_ZERO_ERROR;
+        resultlength=resultLengthOut+1;
+        result=(UChar*)malloc(sizeof(UChar) * resultlength);
+        u_formatMessage( "fr", pattern, u_strlen(pattern), result, resultlength, &status, str , str1);
+        if(u_strcmp(result, expected)==0)
+            log_verbose("PASS: MessagFormat successful on Select test#2\n");
+        else{
+            log_err("FAIL: Error in MessageFormat on Select test#2\n GOT %s EXPECTED %s\n", austrdup(result),
+                austrdup(expected) );
+        }
+        free(result);
+    }
+    if(U_FAILURE(status)){
+        log_data_err("ERROR: failure in message format on Select test#2 : %s \n", myErrorName(status));
+    }
+    free(str);
+    free(str1);
+}
+
 /* test message format with a choice option */
 static void TestMsgFormatChoice(void)
 {
@@ -682,7 +748,7 @@ static void TestMsgFormatChoice(void)
         free(result);
     }
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in message format on test#6 : %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in message format on test#6 : %s (Are you missing data?)\n", myErrorName(status));
     }
 
     log_verbose("Testing message format with choice test #7\n:");
@@ -705,7 +771,7 @@ static void TestMsgFormatChoice(void)
         free(result);
     }
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in message format on test#7 : %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in message format on test#7 : %s (Are you missing data?)\n", myErrorName(status));
     }
 
     log_verbose("Testing message format with choice test #8\n:");
@@ -729,7 +795,7 @@ static void TestMsgFormatChoice(void)
         free(result);
     }
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in message format on test#8 : %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in message format on test#8 : %s (Are you missing data?)\n", myErrorName(status));
     }
 
     free(str);
@@ -754,9 +820,9 @@ static void TestParseMessage(void)
         
     u_parseMessage( "en_US", pattern, u_strlen(pattern), source, u_strlen(source), &status, str, &value);
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in parse Message on test#9: %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in parse Message on test#9: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(value==500.00  && u_strcmp(str,res)==0)
+    else if(value==500.00  && u_strcmp(str,res)==0)
         log_verbose("PASS: parseMessage successful on test#9\n");
     else
         log_err("FAIL: Error in parseMessage on test#9 \n");
@@ -771,15 +837,12 @@ static void TestParseMessage(void)
         
     u_parseMessage( "en_US", pattern, u_strlen(pattern), source, u_strlen(source), &status, &value, str);
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in parse Message on test#10: %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in parse Message on test#10: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(value==123.00 && u_strcmp(str,res)==0)
+    else if(value==123.00 && u_strcmp(str,res)==0)
         log_verbose("PASS: parseMessage successful on test#10\n");
     else
         log_err("FAIL: Error in parseMessage on test#10 \n");
-
-
-    
 }
 
 static int32_t CallFormatMessage(const char* locale, UChar* testCasePattern, int32_t patternLength, 
@@ -825,9 +888,9 @@ static void TestMessageFormatWithValist( void )
                 &status, 1, 3456.00, d1);
         }
         if(U_FAILURE(status)){
-            log_err("ERROR: failure in message format on testcase %d:  %s\n", i, myErrorName(status) );
+            log_data_err("ERROR: failure in message format on testcase %d:  %s (Are you missing data?)\n", i, myErrorName(status) );
         }
-        if(u_strcmp(result, testResultStrings[i])==0){
+        else if(u_strcmp(result, testResultStrings[i])==0){
             log_verbose("PASS: MessagFormat successful on testcase : %d\n", i);
         }
         else{
@@ -869,12 +932,12 @@ static void TestParseMessageWithValist(void)
 
     CallParseMessage( "en_US", pattern, u_strlen(pattern), source, u_strlen(source), &status, str, &value);
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in parse Message on test#9: %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in parse Message on test#9: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(value==500.00  && u_strcmp(str,res)==0)
+    else if(value==500.00  && u_strcmp(str,res)==0)
         log_verbose("PASS: parseMessage successful on test#9\n");
     else
-        log_err("FAIL: Error in parseMessage on test#9 \n");
+        log_err("FAIL: Error in parseMessage on test#9\n");
 
 
     log_verbose("\nTesting a sample for parse Message test#10\n");
@@ -885,9 +948,9 @@ static void TestParseMessageWithValist(void)
 
     CallParseMessage( "en_US", pattern, u_strlen(pattern), source, u_strlen(source), &status, &value, str);
     if(U_FAILURE(status)){
-        log_err("ERROR: failure in parse Message on test#10: %s\n", myErrorName(status));
+        log_data_err("ERROR: failure in parse Message on test#10: %s (Are you missing data?)\n", myErrorName(status));
     }
-    if(value==123.00 && u_strcmp(str,res)==0)
+    else if(value==123.00 && u_strcmp(str,res)==0)
         log_verbose("PASS: parseMessage successful on test#10\n");
     else
         log_err("FAIL: Error in parseMessage on test#10 \n");    
@@ -929,7 +992,7 @@ static void TestJ904(void) {
     if (strcmp(cresult, EXP) == 0) {
         log_verbose("Ok: \"%s\"\n", cresult);
     } else {
-        log_err("FAIL: got \"%s\", expected \"%s\"\n", cresult, EXP);
+        log_data_err("FAIL: got \"%s\", expected \"%s\" -> %s (Are you missing data?)\n", cresult, EXP, u_errorName(status));
     }
 
     ctest_resetTimeZone();
@@ -991,13 +1054,13 @@ static void OpenMessageFormatTest(void)
     umsg_applyPattern(f1,pattern,(int32_t)strlen(PAT),NULL,&status);
     if(U_FAILURE(status))
     {
-        log_err("umsg_applyPattern failed. Error %s \n",u_errorName(status));
+        log_data_err("umsg_applyPattern failed. Error %s (Are you missing data?)\n",u_errorName(status));
     }
 
     /* Test umsg_toPattern              */
     umsg_toPattern(f1,result,256,&status);
     if(U_FAILURE(status) ){
-        log_err("umsg_toPattern method failed. Error: %s \n",u_errorName(status));
+        log_data_err("umsg_toPattern method failed. Error: %s (Are you missing data?)\n",u_errorName(status));
     } else {
         if(u_strcmp(result,pattern)!=0){
             u_UCharsToChars(result,cresult,256);
@@ -1038,13 +1101,37 @@ static void TestErrorChaining(void) {
 
     umsg_open(NULL, 0, NULL, NULL, &status);
     umsg_applyPattern(NULL, NULL, 0, NULL, &status);
+    umsg_toPattern(NULL, NULL, 0, &status);
     umsg_clone(NULL, &status);
+    umsg_format(NULL, NULL, 0, &status);
+    umsg_parse(NULL, NULL, 0, NULL, &status);
     umsg_close(NULL);
 
     /* All of this code should have done nothing. */
     if (status != U_USELESS_COLLATOR_ERROR) {
         log_err("Status got changed to %s\n", u_errorName(status));
     }
+
+    status = U_ZERO_ERROR;
+    umsg_open(NULL, 0, NULL, NULL, &status);
+    if (status != U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("Status should be U_ILLEGAL_ARGUMENT_ERROR instead of %s\n", u_errorName(status));
+    }
+    status = U_ZERO_ERROR;
+    umsg_applyPattern(NULL, NULL, 0, NULL, &status);
+    if (status != U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("Status should be U_ILLEGAL_ARGUMENT_ERROR instead of %s\n", u_errorName(status));
+    }
+    status = U_ZERO_ERROR;
+    umsg_toPattern(NULL, NULL, 0, &status);
+    if (status != U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("Status should be U_ILLEGAL_ARGUMENT_ERROR instead of %s\n", u_errorName(status));
+    }
+    status = U_ZERO_ERROR;
+    umsg_clone(NULL, &status);
+    if (status != U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("Status should be U_ILLEGAL_ARGUMENT_ERROR instead of %s\n", u_errorName(status));
+    }
 }
 
 void addMsgForTest(TestNode** root);
@@ -1064,6 +1151,7 @@ void addMsgForTest(TestNode** root)
     addTest(root, &TestJ904, "tsformat/cmsgtst/TestJ904");
     addTest(root, &MessageLength, "tsformat/cmsgtst/MessageLength");
     addTest(root, &TestErrorChaining, "tsformat/cmsgtst/TestErrorChaining");
+    addTest(root, &TestMsgFormatSelect, "tsformat/cmsgtst/TestMsgFormatSelect");
 }
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
