@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1998-2007, International Business Machines Corporation and
+ * Copyright (c) 1998-2010, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /*
@@ -48,13 +48,18 @@
 #include "ucnv_io.h"
 #include "uprops.h"
 #include "ucase.h"
+#include "ucol_imp.h"
 #include "ucol_swp.h"
 #include "ucnv_bld.h"
-#include "unormimp.h"
 #include "sprpimpl.h"
 #include "propname.h"
 #include "rbbidata.h"
 
+U_CAPI int32_t U_EXPORT2
+unorm2_swap(const UDataSwapper *ds,
+            const void *inData, int32_t length, void *outData,
+            UErrorCode *pErrorCode);
+
 /* other definitions and prototypes */
 
 #define LENGTHOF(array) (int32_t)(sizeof(array)/sizeof((array)[0]))
@@ -79,16 +84,18 @@ void addUDataTest(TestNode** root);
 void
 addUDataTest(TestNode** root)
 {
+#if !UCONFIG_NO_FILE_IO && !UCONFIG_NO_LEGACY_CONVERSION
     addTest(root, &TestUDataOpen,       "udatatst/TestUDataOpen"      );
     addTest(root, &TestUDataOpenChoiceDemo1, "udatatst/TestUDataOpenChoiceDemo1");
     addTest(root, &TestUDataOpenChoiceDemo2, "udatatst/TestUDataOpenChoiceDemo2"); 
     addTest(root, &TestUDataGetInfo,    "udatatst/TestUDataGetInfo"   );
     addTest(root, &TestUDataGetMemory,  "udatatst/TestUDataGetMemory" );
-    addTest(root, &TestUDataSetAppData, "udatatst/TestUDataSetAppData" );
     addTest(root, &TestErrorConditions, "udatatst/TestErrorConditions");
     addTest(root, &TestAppData, "udatatst/TestAppData" );
-    addTest(root, &TestICUDataName, "udatatst/TestICUDataName" );
     addTest(root, &TestSwapData, "udatatst/TestSwapData" );
+#endif
+    addTest(root, &TestUDataSetAppData, "udatatst/TestUDataSetAppData" );
+    addTest(root, &TestICUDataName, "udatatst/TestICUDataName" );
     addTest(root, &PointerTableOfContents, "udatatst/PointerTableOfContents" );
     addTest(root, &SetBadCommonData, "udatatst/SetBadCommonData" );
     addTest(root, &TestUDataFileAccess, "udatatst/TestUDataFileAccess" );
@@ -132,7 +139,8 @@ static void TestUDataOpen(){
     
     const char* testPath=loadTestData(&status);
     if(U_FAILURE(status)) {
-        log_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        free(path);
         return;
     }
 
@@ -143,7 +151,7 @@ static void TestUDataOpen(){
     log_verbose("Testing udata_open()\n");
     result=udata_open(testPath, type, name, &status);
     if(U_FAILURE(status)){
-        log_err("FAIL: udata_open() failed for path = %s, name=%s, type=%s, \n errorcode=%s\n", testPath, name, type, myErrorName(status));
+        log_data_err("FAIL: udata_open() failed for path = %s, name=%s, type=%s, \n errorcode=%s\n", testPath, name, type, myErrorName(status));
     } else {
         log_verbose("PASS: udata_open worked\n");
         udata_close(result);
@@ -199,10 +207,10 @@ static void TestUDataOpen(){
     strcat(icuDataFilePath, dirSepString);
     strcat(icuDataFilePath, U_ICUDATA_NAME);
     strcat(icuDataFilePath, "_");
-    strcat(icuDataFilePath, "unorm.icu");
+    strcat(icuDataFilePath, "cnvalias.icu");
 
     /* lots_of_mallocs(); */
-/*    if (stat(icuDataFilePath, &stat_buf) == 0)*/
+    if (stat(icuDataFilePath, &stat_buf) == 0)
     {
         int i;
         log_verbose("%s exists, so..\n", icuDataFilePath);
@@ -222,11 +230,11 @@ static void TestUDataOpen(){
             }
         }
     }
-/*    else
+    else
     {
          log_verbose("Skipping tests of udata_open() on %s.  File not present in this configuration.\n",
              icuDataFilePath);
-    }*/
+    }
 
     free(icuDataFilePath);
     icuDataFilePath = NULL;
@@ -247,7 +255,7 @@ static void TestUDataOpen(){
     status = U_ZERO_ERROR;
     result = udata_open( testPath, "typ", "nam", &status);
     if (status != U_ZERO_ERROR) {
-        log_err("FAIL: udata_open( \"%s\", \"typ\", \"nam\") returned status %s\n", testPath, u_errorName(status));
+        log_data_err("FAIL: udata_open( \"%s\", \"typ\", \"nam\") returned status %s\n", testPath, u_errorName(status));
     }
     udata_close(result);
     free(icuDataFilePath);
@@ -313,7 +321,7 @@ static void TestUDataOpen(){
         strcat(longTestPath, testPath);
         result=udata_open(longTestPath, type, name, &status);
         if(U_FAILURE(status)){
-            log_err("FAIL: udata_open() failed for path = %s\n name=%s, type=%s, \n errorcode=%s\n",
+            log_data_err("FAIL: udata_open() failed for path = %s\n name=%s, type=%s, \n errorcode=%s\n",
                 longTestPath, name, type, myErrorName(status));
         } else {
             log_verbose("PASS: udata_open worked\n");
@@ -332,7 +340,7 @@ static void TestUDataOpen(){
 
         result=udata_open(longTestPath, type, longName, &status);
         if (status != U_FILE_ACCESS_ERROR) {
-            log_err("FAIL: udata_open() failed for path = %s\n name=%s, type=%s, \n errorcode=%s\n",
+            log_data_err("FAIL: udata_open() failed for path = %s\n name=%s, type=%s, \n errorcode=%s\n",
                 longTestPath, longName, type, myErrorName(status));
         }
         udata_close(result);
@@ -496,9 +504,10 @@ static void TestUDataFileAccess(){
     u_setDataDirectory(icuDataDir);
     u_init(&status);
     if(U_FAILURE(status)){
-        log_err("%s\n", u_errorName(status));
+        log_err_status(status, "%s\n", u_errorName(status));
     }
     free(icuDataDir);
+    ctest_resetICU();
 }
 
 
@@ -601,13 +610,13 @@ static void TestUDataOpenChoiceDemo1() {
     const char* testPath="testdata";
     const char* fullTestDataPath = loadTestData(&status);
     if(U_FAILURE(status)) {
-        log_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
         return;
     }
 
     result=udata_openChoice(NULL, "icu", name[0], isAcceptable1, NULL, &status);
     if(U_FAILURE(status)){
-        log_err("FAIL: udata_openChoice() failed name=%s, type=%s, \n errorcode=%s\n", name[0], type, myErrorName(status));
+        log_data_err("FAIL: udata_openChoice() failed name=%s, type=%s, \n errorcode=%s\n", name[0], type, myErrorName(status));
     } else {
         log_verbose("PASS: udata_openChoice worked\n");
         udata_close(result);
@@ -619,7 +628,7 @@ static void TestUDataOpenChoiceDemo1() {
         status=U_ZERO_ERROR;
         result=udata_openChoice(NULL, type, name[1], isAcceptable2, NULL, &status);
         if(U_FAILURE(status)){
-            log_err("FAIL: udata_openChoice() failed name=%s, type=%s, \n errorcode=%s\n", name[1], type, myErrorName(status));
+            log_data_err("FAIL: udata_openChoice() failed name=%s, type=%s, \n errorcode=%s\n", name[1], type, myErrorName(status));
         }
     }
     else {
@@ -636,7 +645,7 @@ static void TestUDataOpenChoiceDemo1() {
         status=U_ZERO_ERROR;
         result=udata_openChoice(testPath, type, name[2], isAcceptable3, NULL, &status);
         if(U_FAILURE(status)){
-            log_err("FAIL: udata_openChoice() failed path=%s name=%s, type=%s, \n errorcode=%s\n", testPath, name[2], type, myErrorName(status));
+            log_data_err("FAIL: udata_openChoice() failed path=%s name=%s, type=%s, \n errorcode=%s\n", testPath, name[2], type, myErrorName(status));
         }
     }
     else {
@@ -697,13 +706,13 @@ static void TestUDataOpenChoiceDemo2() {
     const char* type="icu";
     const char* path = loadTestData(&status);
     if(U_FAILURE(status)) {
-        log_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
         return;
     }
 
     result=udata_openChoice(path, type, name, isAcceptable, &p, &status);
     if(U_FAILURE(status)){
-        log_err("failed to load data at p=%s t=%s n=%s, isAcceptable", path, type, name);
+        log_data_err("failed to load data at p=%s t=%s n=%s, isAcceptable", path, type, name);
     }
     if(U_SUCCESS(status) ) {
         udata_close(result);
@@ -719,12 +728,12 @@ static void TestUDataOpenChoiceDemo2() {
                 p++;
             }
             else {
-                log_err("FAIL: failed to either load the data or to reject the loaded data. ERROR=%s\n", myErrorName(status) );
+                log_data_err("FAIL: failed to either load the data or to reject the loaded data. ERROR=%s\n", myErrorName(status) );
             }
         }
         else if(p == 2) {
             if(U_FAILURE(status)) {
-                log_err("FAIL: failed to load the data and accept it.  ERROR=%s\n", myErrorName(status) );
+                log_data_err("FAIL: failed to load the data and accept it.  ERROR=%s\n", myErrorName(status) );
             }
             else {
                 log_verbose("Loads the data and accepts it for p==2 as expected\n");
@@ -759,14 +768,14 @@ static void TestUDataGetInfo() {
 
     const char* testPath=loadTestData(&status);
     if(U_FAILURE(status)) {
-        log_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
         return;
     }
 
     log_verbose("Testing udata_getInfo() for cnvalias.icu\n");
     result=udata_open(NULL, "icu", name, &status);
     if(U_FAILURE(status)){
-        log_err("FAIL: udata_open() failed for path = NULL, name=%s, type=%s, \n errorcode=%s\n",  name, type, myErrorName(status));
+        log_data_err("FAIL: udata_open() failed for path = NULL, name=%s, type=%s, \n errorcode=%s\n",  name, type, myErrorName(status));
         return;
     }
     udata_getInfo(result, &dataInfo);
@@ -790,7 +799,7 @@ static void TestUDataGetInfo() {
     log_verbose("Testing udata_getInfo() for test.icu\n");
     result=udata_open(testPath, type, name2, &status);
     if(U_FAILURE(status)) {
-       log_err("FAIL: udata_open() failed for path=%s name2=%s, type=%s, \n errorcode=%s\n", testPath, name2, type, myErrorName(status));
+       log_data_err("FAIL: udata_open() failed for path=%s name2=%s, type=%s, \n errorcode=%s\n", testPath, name2, type, myErrorName(status));
        return;
     }
     udata_getInfo(result, &dataInfo);
@@ -825,7 +834,7 @@ static void TestUDataGetMemory() {
 
     const char* testPath = loadTestData(&status);
     if(U_FAILURE(status)) {
-        log_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
         return;
     }
 
@@ -833,7 +842,7 @@ static void TestUDataGetMemory() {
     log_verbose("Testing udata_getMemory() for \"cnvalias.icu\"\n");
     result=udata_openChoice(NULL, type, name, isAcceptable1, NULL, &status);
     if(U_FAILURE(status)){
-        log_err("FAIL: udata_openChoice() failed for name=%s, type=%s, \n errorcode=%s\n", name, type, myErrorName(status));
+        log_data_err("FAIL: udata_openChoice() failed for name=%s, type=%s, \n errorcode=%s\n", name, type, myErrorName(status));
         return;
     }
     table=(const int32_t *)udata_getMemory(result);
@@ -848,7 +857,7 @@ static void TestUDataGetMemory() {
     log_verbose("Testing udata_getMemory for \"test.icu\"()\n");
     result=udata_openChoice(testPath, type, name2, isAcceptable3, NULL, &status);
     if(U_FAILURE(status)){
-        log_err("FAIL: udata_openChoice() failed for path=%s name=%s, type=%s, \n errorcode=%s\n", testPath, name2, type, myErrorName(status));
+        log_data_err("FAIL: udata_openChoice() failed for path=%s name=%s, type=%s, \n errorcode=%s\n", testPath, name2, type, myErrorName(status));
         return;
     }
     intValue=(uint16_t *)udata_getMemory(result);
@@ -884,7 +893,7 @@ static void TestErrorConditions(){
 
     const char *testPath = loadTestData(&status);
     if(U_FAILURE(status)) {
-        log_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
         return;
     }
 
@@ -893,7 +902,7 @@ static void TestErrorConditions(){
     log_verbose("Testing udata_open() with status != U_ZERO_ERROR\n");
     result=udata_open(testPath, type, name, &status);
     if(result != NULL){
-        log_err("FAIL: udata_open() is supposed to fail for path = %s, name=%s, type=%s, \n errorcode !=U_ZERO_ERROR\n", testPath, name, type);
+        log_data_err("FAIL: udata_open() is supposed to fail for path = %s, name=%s, type=%s, \n errorcode !=U_ZERO_ERROR\n", testPath, name, type);
         udata_close(result);
 
     } else {
@@ -993,14 +1002,14 @@ static void TestAppData()
 
     const char* testPath=loadTestData(&status);
     if(U_FAILURE(status)) {
-        log_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
         return;
     }
 
     icu = ures_open(NULL, "root", &status);
     if(U_FAILURE(status))
     { 
-        log_err("%s:%d: Couldn't open root ICU bundle- %s", __FILE__, __LINE__, u_errorName(status));
+        log_data_err("%s:%d: Couldn't open root ICU bundle- %s", __FILE__, __LINE__, u_errorName(status));
         return;
     }
     /*  log_info("Open icu root: %s size_%d\n", u_errorName(status), ures_getSize(icu)); */
@@ -1009,7 +1018,7 @@ static void TestAppData()
     app = ures_open(testPath, "root", &status);
     if(U_FAILURE(status))
     { 
-        log_err("%s:%d: Couldn't open app ICU bundle [%s]- %s", __FILE__, __LINE__, testPath, u_errorName(status));
+        log_data_err("%s:%d: Couldn't open app ICU bundle [%s]- %s", __FILE__, __LINE__, testPath, u_errorName(status));
         return;
     }
     /* log_info("Open  app: %s, size %d\n", u_errorName(status), ures_getSize(app)); */
@@ -1251,10 +1260,10 @@ static const struct {
     /* Test a 32-bit key table. This is large. */
     {"*testtable32",             "res", ures_swap},
 
-    /* ICU 2.6 resource bundle - data format 1.0, without indexes[] (little-endian ASCII) */
-    {"*icu26_testtypes",         "res", ures_swap},
+    /* ICU 4.2 resource bundle - data format 1.2 (little-endian ASCII) */
+    {"*old_l_testtypes",         "res", ures_swap},
     /* same for big-endian EBCDIC */
-    {"*icu26e_testtypes",        "res", ures_swap},
+    {"*old_e_testtypes",         "res", ures_swap},
 
 #if !UCONFIG_NO_COLLATION
     /* standalone collation data files */
@@ -1281,7 +1290,7 @@ static const struct {
      * MBCS conversion table file without extension,
      * to test swapping and preflighting of UTF-8-friendly mbcsIndex[].
      */
-    {"jisx-208",                 "cnv", ucnv_swap},
+    {"jisx-212",                 "cnv", ucnv_swap},
 #endif
 
 #if !UCONFIG_NO_CONVERSION
@@ -1290,7 +1299,7 @@ static const struct {
 #endif
 
 #if !UCONFIG_NO_IDNA
-    {"uidna",                    "spp", usprep_swap},
+    {"rfc3491",                    "spp", usprep_swap},
 #endif
 
 #if !UCONFIG_NO_BREAK_ITERATION
@@ -1312,13 +1321,15 @@ static const struct {
      * we need not jump through hoops (like adding snapshots of these files
      * to testdata) for code coverage in tests.
      * See Jitterbug 4497.
+     *
+     * ICU4C 4.4 adds normalization data files again, e.g., nfc.nrm.
      */
-#if !UCONFIG_NO_NORMALIZATION
-    {"unorm",                    "icu", unorm_swap},
-#endif
     {"uprops",                   "icu", uprops_swap},
     {"ucase",                    "icu", ucase_swap},
     {"ubidi",                    "icu", ubidi_swap},
+#endif
+#if !UCONFIG_NO_NORMALIZATION
+    {"nfc",                      "nrm", unorm2_swap},
 #endif
     {"unames",                   "icu", uchar_swapNames}
 };
@@ -1566,7 +1577,7 @@ TestSwapData() {
     UErrorCode errorCode = U_ZERO_ERROR;
     int32_t i;
 
-    buffer=(uint8_t *)uprv_malloc(2*SWAP_BUFFER_SIZE);
+    buffer=(uint8_t *)malloc(2*SWAP_BUFFER_SIZE);
     if(buffer==NULL) {
         log_err("unable to allocate %d bytes\n", 2*SWAP_BUFFER_SIZE);
         return;
@@ -1574,7 +1585,7 @@ TestSwapData() {
 
     testPath=loadTestData(&errorCode);
     if(U_FAILURE(errorCode)) {
-        log_err("Could not load testdata.dat, status = %s\n", u_errorName(errorCode));
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(errorCode));
     }
 
     /* Test that printError works as expected. */
@@ -1642,6 +1653,11 @@ TestSwapData() {
             pkg=U_ICUDATA_BRKITR;
             nm=swapCases[i].name;
             uprv_strcpy(name, U_ICUDATA_BRKITR);
+        } else if (uprv_strcmp(swapCases[i].name, "ucadata")==0
+            || uprv_strcmp(swapCases[i].name, "invuca")==0) {
+            pkg=U_ICUDATA_COLL;
+            nm=swapCases[i].name;
+            uprv_strcpy(name, U_ICUDATA_COLL);
         } else {
             pkg=NULL;
             nm=swapCases[i].name;
@@ -1662,7 +1678,7 @@ TestSwapData() {
         }
     }
 
-    uprv_free(buffer);
+    free(buffer);
 }
 
 
@@ -1738,18 +1754,6 @@ static void SetBadCommonData(void) {
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
         log_err("FAIL: udata_setCommonData did not fail with bad arguments.\n");
     }
-    if (u_getDataDirectory() == NULL || u_getDataDirectory()[0] == 0) {
-        /* Check that we don't change the common data. Many ICU tests will fail after this test, if this works. */
-        /* We could do a better test, if we called u_cleanup and udata_setCommonData returned the last value. */
-        status = U_ZERO_ERROR;
-        udata_setCommonData(&gOffsetTOCAppData_dat, &status);
-        if (status != U_USING_DEFAULT_WARNING) {
-            log_err("FAIL: udata_setCommonData allowed the data to be changed after initialization!\n");
-        }
-    }
-    else {
-        log_verbose("Can't test setting common data because files mode may have been used.\n");
-    }
 
     /* Check that we verify that the data isn't bad */
     status = U_ZERO_ERROR;
