@@ -1,6 +1,6 @@
 
 /********************************************************************
- * Copyright (c) 1997-2007, International Business Machines
+ * Copyright (c) 1997-2009, International Business Machines
  * Corporation and others. All Rights Reserved.
  ********************************************************************/
  
@@ -88,12 +88,21 @@ public:
 
     void TestFebruary(void);
 
+    void TestCanonicalID(void);
+    
+    virtual void TestDisplayNamesMeta();
+
     static const UDate INTERVAL;
 
 private:
     // internal functions
-    static UnicodeString& formatMinutes(int32_t min, UnicodeString& rv, UBool insertSep = TRUE);
-    static UnicodeString& formatRFC822TZ(int32_t min, UnicodeString& rv);
+    static UnicodeString& formatOffset(int32_t offset, UnicodeString& rv);
+    static UnicodeString& formatTZID(int32_t offset, UnicodeString& rv);
+
+    // Some test case data is current date/tzdata version sensitive and producing errors
+    // when year/rule are changed.
+    static const int32_t REFERENCE_YEAR;
+    static const char *REFERENCE_DATA_VERSION;
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
